#include <config.h>

#include "antilink.h"
#include "safeio.h"

#ifdef WITH_DMALLOC
# include <dmalloc.h>
#endif

static unsigned int zrand(void)
{
    int fd;
    int ret;
    
    if (
#ifdef HAVE_DEV_ARANDOM
        ((fd = open("/dev/arandom", O_RDONLY | O_NONBLOCK)) == -1)
#elif defined(HAVE_DEV_URANDOM)
        ((fd = open("/dev/urandom", O_RDONLY | O_NONBLOCK)) == -1)
#else
        ((fd = open("/dev/random", O_RDONLY | O_NONBLOCK)) == -1)
#endif        
        ) {
        nax:
#ifdef HAVE_RANDOM
        return (unsigned int) random();        
#else
        return (unsigned int) rand();
#endif
    }
    if (read(fd, &ret, sizeof ret) != (ssize_t) sizeof ret) {
        close(fd);
        goto nax;
    }
    close(fd);    
    
    return (unsigned int) ret;
}

static void init_zrand(void)
{
    struct timeval tv;
    struct timezone tz;
    
    gettimeofday(&tv, &tz);
#ifdef HAVE_SRANDOMDEV
    srandomdev();
#endif    
#ifdef HAVE_RANDOM
    srandom((unsigned int) (tv.tv_sec ^ tv.tv_usec));
#else
    srand((unsigned int) (tv.tv_sec ^ tv.tv_usec));
#endif
}

int main(void)
{
    int ofd;
    unsigned int rn;
    size_t s;

    (void) close(0);
    if ((ofd = open(ANTILINK_KEY_FILE, 
                    O_CREAT | O_WRONLY | O_EXCL | O_NOFOLLOW, 0644)) == -1) {
        perror(ANTILINK_KEY_FILE);
        return 1;
    }
    init_zrand();    
    s = (size_t) 0U;
    do {
        rn = zrand();
        if (safe_write(ofd, &rn, sizeof rn) != 0) {
            perror(ANTILINK_KEY_FILE);
            (void) close(ofd);
            return 2;
        }
        s += sizeof rn;
    } while (s < ANTILINK_KEY_LEN);
    (void) close(ofd);
    
    return 0;
}
